@@ -16,18 +16,13 @@
 static git_repository *repo;
 
 static const char *relpath = "";
-static const char *repodir;
 
 static char description[255] = "Repositories";
 static char *name = "";
 static char owner[255];
 
 #ifndef USE_PLEDGE
-int
-pledge(const char *promises, const char *paths[])
-{
-        return 0;
-}
+#define pledge(p1,p2) 0
 #endif
 
 void
@@ -52,7 +47,7 @@ xmlencode(FILE *fp, const char *s, size_t len)
 		switch(*s) {
 		case '<':  fputs("&lt;",   fp); break;
 		case '>':  fputs("&gt;",   fp); break;
-		case '\'': fputs("&apos;", fp); break;
+		case '\'': fputs("&#39;" , fp); break;
 		case '&':  fputs("&amp;",  fp); break;
 		case '"':  fputs("&quot;", fp); break;
 		default:   fputc(*s, fp);
@@ -78,20 +73,22 @@ void
 writeheader(FILE *fp)
 {
 	fputs("<!DOCTYPE html>\n"
-		"<html dir=\"ltr\" lang=\"en\">\n<head>\n"
+		"<html>\n<head>\n"
 		"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n"
-		"<meta http-equiv=\"Content-Language\" content=\"en\" />\n<title>", fp);
+		"<title>", fp);
 	xmlencode(fp, description, strlen(description));
 	fprintf(fp, "</title>\n<link rel=\"icon\" type=\"image/png\" href=\"%sfavicon.png\" />\n", relpath);
 	fprintf(fp, "<link rel=\"stylesheet\" type=\"text/css\" href=\"%sstyle.css\" />\n", relpath);
 	fputs("</head>\n<body>\n", fp);
 	fprintf(fp, "<table>\n<tr><td><img src=\"%slogo.png\" alt=\"\" width=\"32\" height=\"32\" /></td>\n"
-	        "<td><h1>%s</h1><span class=\"desc\">%s</span></td></tr><tr><td></td><td>\n",
-		relpath, name, description);
-	fputs("</td></tr>\n</table>\n<hr/>\n<div id=\"content\">\n"
-	      "<table id=\"index\"><thead>\n"
-	      "<tr><td>Name</td><td>Description</td><td>Owner</td><td>Last commit</td></tr>"
-	      "</thead><tbody>\n", fp);
+	        "<td><span class=\"desc\">", relpath);
+	xmlencode(fp, description, strlen(description));
+	fputs("</span></td></tr><tr><td></td><td>\n"
+		"</td></tr>\n</table>\n<hr/>\n<div id=\"content\">\n"
+		"<table id=\"index\"><thead>\n"
+		"<tr><td><b>Name</b></td><td><b>Description</b></td><td><b>Owner</b></td>"
+		"<td><b>Last commit</b></td></tr>"
+		"</thead><tbody>\n", fp);
 }
 
 void
@@ -157,6 +154,7 @@ main(int argc, char *argv[])
 	const git_error *e = NULL;
 	FILE *fp;
 	char path[PATH_MAX], repodirabs[PATH_MAX + 1];
+	const char *repodir;
 	int i, ret = 0;
 
 	if (pledge("stdio rpath", NULL) == -1)
